########################################################################################
# Functions
########################################################################################
myRGBColVec <- function (nrgcols = 12) {
    k <- trunc(nrgcols/2)
    if (2 * k == nrgcols) {
        r <- c(rev(seq(0, 1, length = k)), rep(0, k))
        g <- c(rep(0, k), seq(0, 1, length = k))
    #change
        #colvec <- rgb(r, g, rep(0, 2 * k))
        colvec <- rgb(g, r, rep(0, 2 * k))
    }
    else {
        r <- c(rev(seq(1/(2 * k - 1), 1, length = k)), rep(0, 
            k + 1))
        g <- c(rep(0, k + 1), seq(1/(2 * k - 1), 1, length = k))
        #colvec <- rgb(r, g, rep(0, 2 * k + 1))
        colvec <- rgb(g, r, rep(0, 2 * k + 1))
    }

    colvec
}

blueYellowBColVec <- function (nrgcols = 12) {
    k <- trunc(nrgcols/2)
    if (2 * k == nrgcols) {
        r <- c(rep(0, k), seq(0, 1, length = k))
        g <- r
        b <- c(rev(seq(0, 1, length = k)), rep(0, k))
    #change
        #colvec <- rgb(r, g, b, rep(0, 2 * k))
        colvec <- rgb(r, g, b)
    }
    else {
        r <- c(rep(0, k + 1), seq(1/(2 * k - 1), 1, length = k))
        g <- r
        b <- c(rev(seq(1/(2 * k - 1), 1, length = k)), rep(0, 
            k + 1))
        #colvec <- rgb(r, g, b, rep(0, 2 * k + 1))
    colvec <- rgb(r, g, b)
    
    }
    colvec
}

# heatmap top color labels
changeColorGroups <- function(inputColors) {
  ncolors <- inputColors
  
  for (k in 1:length(ncolors)) {
    if (identical(ncolors[k], "red")) { 
      ncolors[k] <- "black" #blue"
    } else 
    
    if (identical(ncolors[k], "blue")) {
      ncolors[k] <- "grey" #yellow"
    }
  }
  
  ncolors
}


whiteColumn2IngoreLast <- function(inputColors) {
  ncolors <- inputColors
  rows <- length(ncolors[, 2]) - 1
  
  for (k in 1:rows) {
    ncolors[k, 2] <- "white"
    ncolors[k, 1] <- "white"
  }
  
  ncolors
}

whiteIngoreOddAndLast <- function(inputColors) {
  ncolors <- inputColors
  
  for (k in 1:(length(ncolors) - 1)) {
    if (k %% 2 == 0) {
      ncolors[k] <- "white"
    }
  }
  
  ncolors
}

wght.mn <- function(x){
    flag <- x != mean(x)
    if(length(x) == length(x[flag])){
        w <- 1/(x - mean(x))^2
        weighted.mean(x,w)
    } else {
        mean(x)
    }
}

# Convert dist results to matrix and removed upper-right mirrored data
convertDistToMatrix <- function(distresults) {
  dist.matrix <- as.matrix(distresults)

  for (r in 1:length(dist.matrix[,1])) {
    for (c in 1:length(dist.matrix[1,])) {
      if (c > r) dist.matrix[r,c] <- ""
    }
  }
  
  dist.matrix
}

foldchangdataFile <- function(outfileName, sigdata) {
  filename <- paste(paste(outfileName, "fold_change_data", sep="_"), "txt", sep=".");
  write.table(data.frame('Data_Label' = rownames(sigdata), sigdata), file = filename, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  
  filename
  
}

genedistanceFile <- function(outfileName, distdata) {
  distancelabel <- "gene_cluster_dist"
  filename <- paste(paste(outfileName, distancelabel, sep="_"), "txt", sep=".");

  dist.mt <- convertDistToMatrix(distdata)
  write.table(data.frame('Genes' = rownames(dist.mt), dist.mt), file = filename, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  addDistanceFootnote(filename);
  
  filename
}

chemicaldistanceFile <- function(outfileName, distdata) {
  distancelabel <- "chemical_cluster_dist"
  filename <- paste(paste(outfileName, distancelabel, sep="_"), "txt", sep=".");

  dist.mt <- convertDistToMatrix(distdata)
  write.table(data.frame('Chemicals' = rownames(dist.mt), dist.mt), file = filename, append = FALSE, quote = FALSE, row.names=FALSE, sep = "\t")
  addDistanceFootnote(filename);
  
  filename
}

addDistanceFootnote <- function(distanceFile) {
  dist.note <- "\nValues represent Euclidean distance\n"
  write(dist.note, file=distanceFile, append=TRUE)
}

# fold_change_data, gene_cluster_dist, chemical_cluster_dist
outputfiles <- function(outfilename, fcdata=NA, geneclusterdist=NA, chemcluster=NA, pcresults=NA) {
  # Write data to file
  data_file <- paste(paste(outfileName, "fold_change_data", sep="_"), "txt", sep=".");
  write.table(fcdata, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  
  if (!is.na(geneclusterdist)) {
    gene_cluster_file <- paste(paste(outfileName, "gene_cluster_dist", sep="_"), "txt", sep=".");
    write.table(gene_cluster_file, file = gene_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
  
  if (!is.na(chemcluster)) {{
    pc_file <- paste(paste(outfileName, "chemical_cluster_dist", sep="_"), "txt", sep=".");
    write.table(chemcluster, file = chem_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
  
  if (!is.na(pcresults)) 
    chem_cluster_file <- paste(paste(outfileName, "2D_pc", sep="_"), "txt", sep=".");
    write.table(pcresults, file = chem_cluster_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
  }
}

Affy <- function(Treated, Control, anno, gs, heatData, pctitle, testChemical, outfileName) {
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)


    #my.home <- "Q:/EXCHANGE/MUTAGENESIS/MUTAGENEDATA/Williams/projects 2015/Byron CEBS revised/Affy/"
    #Treated <- "GSM985549_BPI-T2-BaP10-72h-R-1-1.CEL"
    setwd(data_path)
    my.RMA <- justRMA(filenames=Treated)
    dat <- as.data.frame(my.RMA)
    dat <- unlist(dat)
    dat <- data.frame(probe = as.character(names(dat)), Signal = log10(2^as.numeric(dat)))
    
    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Signal), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    out <- dat[flag,]

    #Control <- "GSM985552_BPI-T2-DMSO-72h-R-1-1.CEL"
    setwd(data_path)
    my.RMA <- justRMA(filenames=Control)
    dat <- as.data.frame(my.RMA)
    dat <- unlist(dat)
    dat <- data.frame(probe = as.character(names(dat)), Signal = log10(2^as.numeric(dat)))
    
    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Signal), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    dat <- dat[flag,]

    #Creating the Ratio log10
    out[,2] <- out[,2] - dat[,2]

    #Merge the Classifier
    dat <- merge(gs, out, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17
    time_stamp <- format(Sys.time(), "%d-%b-%Y") # "31-Aug-2016"
    
    #Generate outputs to the defined destination/path
    setwd(results_path)
    # Write data to file
    fcdataFile <- foldchangdataFile(outfileName, x)
    #data_file <- paste(paste(outfileName, "data", sep="_"), "txt", sep=".");
    #write.table(x, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
    genedistfile <- genedistanceFile(outfileName, gene.dist)

    pdf_file = "heatmapAffy.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }

    pdffile <- paste(results_path, pdf_file, sep = "/");
    CairoPDF(file = pdffile, width = 11, height = 8.5,
        family = "Courier")

    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)

    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)

    # Column cluster
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistfile <- chemicaldistanceFile(outfileName, chem.dist)

    ###
    # Repeat heatmap and cluster plots to files
    ###

    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)
    
    df.files <- data.frame(fcdataFile = fcdataFile, genedistfile = genedistfile, chemicaldistfile = chemicaldistfile,
        pdffile=pdffile, heatmapfile=heatmapimgfile, clusterfile=clusterimgfile)
    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}

AgilentDyeSwap <- function(SampleCy5toControlCy3, SampleCy3toControlCy5, anno, gs, heatData, pctitle, testChemical, outfileName){
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)

    dat <- read.table(paste(data_path, SampleCy5toControlCy3, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("LogRatio", names(dat))[1]

    #Feature Extraction LogRatio = log10(Cy5/Cy3)
    #dat[,g3] <- log10(dat$rProcessedSignal/dat$gProcessedSignal)
    dat <- dat[,c(g1,g2)]
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    out <- dat[flag,]

    #Dye Swap
    dat <- read.table(paste(data_path, SampleCy3toControlCy5, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("LogRatio", names(dat))[1]

    #Feature Extraction LogRatio = log10(Cy5/Cy3)
    #dat[,g3] <- log10(dat$rProcessedSignal/dat$gProcessedSignal)
    dat <- dat[,c(g1,g2)]
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    dat <- dat[flag,]

    out[,2] <- (out[,2] - dat[,2])/2

    dat <- merge(gs, out, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17

    setwd(results_path);
    time_stamp <- format(Sys.time(), "%d-%b-%Y")
    genedistfile <- genedistanceFile(outfileName, gene.dist)
    
    # Write data to file
    #data_file <- paste(paste(outfileName, "data", sep="_"), "txt", sep=".");
    #write.table(x, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
    fcdataFile <- foldchangdataFile(outfileName, x)
    
    pdf_file = "heatmapAgilentDyeSwap.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }
    
    CairoPDF(file = paste(results_path, pdf_file, sep = "/"), width = 11, height = 8.5,
        family = "Courier")

    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)

    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistanceFile(outfileName, chem.dist)

    
    ###
    # Repeat heatmap and cluster plots to files
    ###
    
    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)

    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}


AgilentOneCol <- function(Sample, Control, anno, gs, heatData, pctitle, testChemical, outfileName){
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)
    
    #Sample <- "GSM966468_ST3M3376.txt"
    dat <- read.table(paste(data_path, Sample, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("ProcessedSignal", names(dat))[1]

    dat <- dat[,c(g1,g2)]
    dat[,2] <- log10(dat[,2])
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    out <- dat[flag,]

    #Control
    #Control <- "GSM966464_ST1M3346.txt"
    dat <- read.table(paste(data_path, Control, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("ProcessedSignal", names(dat))[1]

    dat <- dat[,c(g1,g2)]
    dat[,2] <- log10(dat[,2])
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    dat <- dat[flag,]

    #logRatio = log10(Sample/Control)
    out[,2] <- out[,2] - dat[,2]

    dat <- merge(gs, out, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17
    setwd(results_path);
    time_stamp <- format(Sys.time(), "%d-%b-%Y")
    fcdataFile <- foldchangdataFile(outfileName, x)
    genedistfile <- genedistanceFile(outfileName, gene.dist)
    
    # Write data to file
    #data_file <- paste(paste(outfileName, "data", sep="_"), "txt", sep=".");
    #write.table(x, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
    
    pdf_file = "heatmapAgilentOneColour.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }
    
    
    CairoPDF(file = paste(results_path, pdf_file, sep = "/"), width = 11, height = 8.5,
        family = "Courier")

    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)
        
    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistanceFile(outfileName, chem.dist)
        
    ###
    # Repeat heatmap and cluster plots to files
    ###
    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)

    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}


AgilentTwoColCy5 <- function(SampleCy5toControlCy3, anno, gs, heatData, pctitle, testChemical, outfileName){
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)

    dat <- read.table(paste(data_path, SampleCy5toControlCy3, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("LogRatio", names(dat))[1]

    #Feature Extraction LogRatio = log10(Cy5/Cy3)
    #dat[,g3] <- log10(dat$rProcessedSignal/dat$gProcessedSignal)
    dat <- dat[,c(g1,g2)]
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    out <- dat[flag,]

    dat <- merge(gs, out, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17
    
    setwd(results_path);
    time_stamp <- format(Sys.time(), "%d-%b-%Y")
    fcdataFile <- foldchangdataFile(outfileName, x)
    genedistfile <- genedistanceFile(outfileName, gene.dist)
    
    # Write data to file
    #data_file <- paste(paste(outfileName, "data", sep="_"), "txt", sep=".");
    #write.table(x, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
    
    pdf_file = "heatmapAgilentTwoColSampleCy5.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }
    
    CairoPDF(file = paste(results_path, pdf_file, sep = "/"), width = 11, height = 8.5,
        family = "Courier")

    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)
        
    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistanceFile(outfileName, chem.dist)

    ###
    # Repeat heatmap and cluster plots to files
    ###
    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)
    
    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}


AgilentTwoColCy3 <- function(SampleCy3toControlCy5, anno, gs, heatData, pctitle, testChemical, outfileName){
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)

    dat <- read.table(paste(data_path, SampleCy3toControlCy5, sep = "/"), header = TRUE, sep = "\t", quote = "",
        fill = TRUE, skip = 9, comment.char = "")
    g1 <- grep("ProbeName", names(dat))
        g2 <- grep("LogRatio", names(dat))[1]

    #Feature Extraction LogRatio = log10(Cy5/Cy3)
    #dat[,g3] <- log10(dat$rProcessedSignal/dat$gProcessedSignal)
    dat <- dat[,c(g1,g2)]
    names(dat) <- c("Probe", "Ratio")

    #Merging Annotation
    dat <- merge(anno, dat, by.x = "ProbeID", by.y = "Probe")

    #Gene Set
    dat <- dat[as.character(dat$GeneSymbol) %in% as.character(gs$ID),]

    #collapsing technical reps  
    dat <- split(as.data.frame(dat$Ratio), as.character(dat$GeneSymbol))
    dat <- lapply(dat, function(x) apply(x, 2, wght.mn))

    probe <- names(dat)
    dat <- matrix(unlist(dat), ncol = 1, byrow = TRUE)
    dat <- cbind.data.frame(probe, dat)
    names(dat) <- c("Probe", "Ratio")
    flag <- order(dat$Probe)
    dat <- dat[flag,]

    #Flipping the ratio
    dat[,2] <- -1 * dat[,2]

    dat <- merge(gs, dat, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17
    
    setwd(results_path);
    time_stamp <- format(Sys.time(), "%d-%b-%Y")
    fcdataFile <- foldchangdataFile(outfileName, x)
    genedistfile <- genedistanceFile(outfileName, gene.dist)
    
    # Write data to file
    #data_file <- paste(paste(outfileName, "data", sep="_"), "txt", sep=".");
    #write.table(x, file = data_file, append = FALSE, quote = FALSE, col.names=NA, sep = "\t")
    
    pdf_file = "heatmapAgilentTwoColSampleCy3.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }
    
    CairoPDF(file = paste(results_path, pdf_file, sep = "/"), width = 11, height = 8.5,
        family = "Courier")

    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)
    
    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistanceFile(outfileName, chem.dist)

    ###
    # Repeat heatmap and cluster plots to files
    ###
    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)
    
    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}

General <- function(dat, gs, heatData, pctitle, testChemical, outfileName){
    library(affy)
    library(Cairo)
    library(heatmap.plus)
    library(Heatplus)
    library(heatmap3)
    library(dendextend)

    dat <- merge(gs, dat, by.x = "ID", by.y = "Probe")

    #Merge Heatmap data
    dat <- merge(dat[,c(1,3,4,7,8)], heatData, by.x = "ID", by.y = "Probe")

    #Arranging the data so that the unknown sample is last
    dat <- dat[,c(1:4, 6:ncol(dat), 5)]

    #Obtain Predictions
    cl <- c(rep("red", 13), rep("blue", 16), "white")
    pred <- rep("white", 29)
    for(k in 5:ncol(dat)){
        G <- sum(((dat[,k] - dat$Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 
        NG <- sum(((dat[,k] - dat$Non.Genotoxic.score)/dat$my.train.sd)^2) - 2*log(0.5) 

        probG <- exp(-G/2)/(exp(-G/2)+exp(-NG/2))
        probNG <- exp(-NG/2)/(exp(-G/2)+exp(-NG/2))
        class <- "Unclassified"
        if(probG > 0.9){
            class <- "Genotoxic"
            pred[k-4] <- "red"
        }
        if(probNG > 0.9){
            class <- "Non-Genotoxic"
            pred[k-4] <- "blue"
        }   
    }
    
    my.col <- cbind(cl, pred)
    x <- as.matrix(dat[,-c(1:4)])
    rownames(x) <- as.character(dat[,1])
    colnames(x) <- c("5 FU",    "AraC",                         "Bleomycin",
                "Camptothecin",     "Cisplatin",                    "Etoposide",                   
        "Hydrogen Peroxide",    "Hydroxyurea",                  "Methotrexate", 
                "MMS",                  "Potassium.Chromate.VI",        "Sodium Arsenite",
                "X ray",        "2 DG",                         "Apicidin",   
                "Antimycin A",          "Cadmium Chloride",             "Colchicine",
                "Docetaxel",            "EtOH 2%",                      "EtOH 4%",
                "HC toxin",             "Heat Shock",                   "Oxamflatin",
                "Paclitaxel",           "Thapsigargin",                 "TSA",                       
            "Tunicamycin",          "Vinblastine",                  testChemical) 

    #Preparing breaks for the heatmap
    end.breaks <- c(0, 29, ncol(x))
    y <- NULL
    my.col2 <- NULL
    g <- NULL
    col.blank <- rep(NA,nrow(x))
    pp <- NULL

    for(k in 2:length(end.breaks)){
        y <- cbind(y, x[, (end.breaks[k-1]+1):end.breaks[k]], col.blank)
        my.col2 <- rbind(my.col2, my.col[(end.breaks[k-1]+1):end.breaks[k],], c("white", "white"))
        g <- c(g, colnames(x)[(end.breaks[k-1]+1):end.breaks[k]], " ")
    }

    y <- y[,-ncol(y)]
    my.col2 <- my.col2[-nrow(my.col2),]
    g <- g[-length(g)]
    colnames(y) <- g

    colnames(my.col) <- c("Class", "Prediction")
    colnames(my.col2) <- c("Class", "Prediction")

    #Row clustering
    gene.dist <- dist(x)
    my.row <- hclust(gene.dist, method = "average")
    my.row <- as.dendrogram(my.row)

    n.col <- 17
    
    setwd(results_path);
    time_stamp <- format(Sys.time(), "%d-%b-%Y")
    fcdataFile <- foldchangdataFile(outfileName, x)
    genedistfile <- genedistanceFile(outfileName, gene.dist)

    pdf_file = "heatmapGeneral.pdf";
    
    if (!is.null(outfileName) && nchar(outfileName) > 0) {
      pdf_file = paste(outfileName, 'pdf', sep = ".");
    }
    
    CairoPDF(file = paste(results_path, pdf_file, sep = "/"), width = 11, height = 8.5,
             pointsize = 12, family = "Courier")
    colColor <-my.col2
    #heatmap3(y, Rowv = my.row, Colv = NA, scale = "row", margins = c(15, 10), 
    #        cexRow=0.7, cexCol=1, col = myRGBColVec(n.col), ColSideColors = my.col2)
    
    plotHeatmap(y, my.row, n.col, my.col2, pctitle, time_stamp)
    chem.dist <- plotCluster(x, pctitle, time_stamp)

    dev.off()
    chemicaldistanceFile(outfileName, chem.dist)

    ###
    # Repeat heatmap and cluster plots to files
    ###
    heatmapimgfile <- heatmapPlotToFile(y, my.row, n.col, my.col2, outfileName, pctitle, time_stamp)
    clusterimgfile <- clusterPlotToFile(x, outfileName, pctitle, time_stamp)
    
    df.clssification <- data.frame(class=class, probG = probG, probNG=probNG)
    
    return (df.clssification)
}

#Heatmap image my.row, n.col, my.col2
heatmap1XPlotToFile <- function(y, rowValue, nCol, colColor, outfileName, pctitle, time_stamp) {
  heatmap_file <- paste(paste(outfileName, "heatmap", sep="_"), 'png', sep = ".");
  #png(filename=heatmap_file, width=1040, height=800, family = "Courier")
  #CairoPNG(filename=heatmap_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=heatmap_file, width=11, height=8.5, unit='in', res=72, type="cairo")
  plotHeatmap(y, rowValue, nCol, colColor, pctitle, time_stamp)
  dev.off()
}

# 2X Heatmap image my.row, n.col, my.col2
heatmapPlotToFile <- function(y, rowValue, nCol, colColor, outfileName, pctitle, time_stamp) {
  heatmap_file <- paste(paste(outfileName, "heatmap", sep="_"), 'png', sep = ".");
  #png(filename=heatmap_file, width=1040, height=800, family = "Courier")
  #CairoPNG(filename=heatmap_file, width=1040, height=720, pointsize = 12, family = "Courier")
  res2X <- 144
  png(filename=heatmap_file, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotHeatmap(y, rowValue, nCol, colColor, pctitle, time_stamp)
  dev.off()
  
  heatmap_file
}

plotHeatmap <- function (y, rowValue, nCol, colColor, pctitle, time_stamp) {
  #color_maps <- myRGBColVec(nCol)
  color_maps <- blueYellowBColVec(nCol)
  col_ch_Colors <- colColor
  col_ch_Colors <- changeColorGroups(colColor)
  #col_ch_Colors <- whiteIngoreLast(col_ch_Colors)
  col_ch_Colors <- whiteColumn2IngoreLast(col_ch_Colors)
 
  class_label <- "NDDI"
  #class_label
  if (col_ch_Colors[31, 2] == "black") {
    class_label <- 'DDI'
  }
  
  col_ch_Colors[31, 2] <- "white"
  
  hc <- heatmap3(y, Rowv = rowValue, Colv = NA, scale = "row", margins = c(15, 10), 
           cexRow=0.65, cexCol=1, col = color_maps, ColSideColors = col_ch_Colors)
  mtext(pctitle, side=3, line=-3, at=c(0.45), cex=1)
  #mtext(c("DNA Damage Inducing (DDI)", "Non DNA Damage Inducing (NDDI)"), side=3, line=-4.5, at=c(0.31, 0.55), cex=0.9)
  mtext(c("DNA Damage Inducing (DDI)", "Non DNA Damage Inducing (NDDI)"), side=3, line=-4.5, at=c(0.24, 0.46), adj=0, cex=0.9)
  mtext(time_stamp, side=1, line=3, cex=0.8)
  mtext(class_label, side=3, line=-5, at=c(0.74), adj=0, cex=0.7)
  
  #left-v
  segments(0.229, 0.84, 0.229, 0.86, col= 'black', lwd=1)
  #mid-v
  segments(0.452, 0.84, 0.452, 0.86, col= 'black', lwd=1)
  #right-v
  segments(0.727, 0.84, 0.727, 0.86, col= 'black', lwd=1)
  #horizental
  segments(0.23, 0.85, 0.726, 0.85, col= 'black', lwd=1)
}

# Cluster image 2X
clusterPlotToFile <- function(x, outfileName, pctitle, time_stamp) {
  cluster_file <- paste(paste(outfileName, "cluster", sep="_"), 'png', sep = ".");
  #png(filename=cluster_file, width=1040, height=800)
  res2X <- 144
  #CairoPNG(filename=cluster_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=cluster_file, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotCluster(x, pctitle, time_stamp)
  dev.off()
  
  cluster_file
}

# Cluster image
clusterPlotToFile0 <- function(x, outfileName, pctitle, time_stamp) {
    cluster_file <- paste(paste(outfileName, "cluster", sep="_"), 'png', sep = ".");
    #png(filename=cluster_file, width=1040, height=800)
    #CairoPNG(filename=cluster_file, width=1040, height=720, pointsize = 12, family = "Courier")
    png(filename=cluster_file, width=11, height=8.5, unit='in', res=72, type="cairo")
    plotCluster(x, pctitle, time_stamp)
    dev.off()
}

plotCluster <- function (x, pctitle, time_stamp) {
  my.text <- colnames(x)
  row.loc <- apply(x[,1:29],1,mean)
  row.scale <- apply(x[,1:29],1,sd)
  x <- (x - row.loc)/row.scale
  
  my.pca <- prcomp(t(x[,1:29]), retx = TRUE, center = FALSE, scale. = FALSE)
  pc1 <- as.vector(my.pca$rotation[,1]) %*% x
  pc2 <- as.vector(my.pca$rotation[,2]) %*% x
  
  #par(mfrow = c(1,2), mar = c(5,4,4,2))
  par(mar = c(5,4,4,2), oma=c(1,1,2,1), ps=12)
  layout(matrix(c(1,2), byrow=T, ncol=2))
  
  small_cex <- 0.5
  large_cex <- small_cex * 3
  plot(pc1, pc2, type = "n", main = "", xlab = "PC 1", ylab = "PC 2")
  points(pc1[1:13], pc2[1:13], pch = 20, cex=small_cex, col = "red")# pch = 16
  points(pc1[14:29], pc2[14:29], pch = 20, cex=small_cex, col = "blue")
  points(pc1[30:ncol(x)], pc2[30:ncol(x)], pch = 17, cex=large_cex, col = "dark green")
  
  text(pc1[1:13], pc2[1:13], labels = my.text[1:13], col = "red", cex = 0.7, pos = 1)
  text(pc1[14:29], pc2[14:29], labels = my.text[14:29], col = "blue", cex = 0.7, pos = 1)
  text(pc1[30:ncol(x)], pc2[30:ncol(x)], labels = my.text[30:ncol(x)], col = "dark green", cex = 0.7, pos = 1)

  #mtext(time_stamp, side=1, line=3, adj=1, cex=0.7)
  
  abline(v = 0, col = "red", lwd = 2)
  
  chem.dist <- dist(t(x))
  
  dendro <- hclust(chem.dist, method = "average")
  dendro <- as.dendrogram(dendro)
  
  # Assigning the labels of dendrogram object with new colors:
  labels_colors(dendro) <- c(rep("red", 13), rep("blue", 16), rep("dark green", 5))[order.dendrogram(dendro)]
  
  par(mar = c(3,1,1,10), cex = 0.7)
  plot(dendro, horiz=T)
  
  mtext(pctitle, side=3, line=-1, outer=T, adj=0.5, cex=1.5)
  mtext(time_stamp, side=1, outer=T, line=-0.5, adj=0.49, cex=0.8)
  mtext("Euclidean distance", side=1, outer=T, line=-4.5, at=(0.7), cex=0.8)
  
  chem.dist
}
