# Batch all columns from the input data file
# Default directories under based dir: Classifier, Code, Data, Results
# Modfied based on tgClassifier.R for batch file general platform
#
# Arugment parameters with example values:
# 1. workPath = "/var/www/html/tgclassifier/rspace"
# 2. sub_dir = "general"
# 3. platform = "general"
# 4. pc_title = "DNA Damage Classification of MJ Chem 1 at 1000 mg/kg"
# 5. test_chemical = "MJ Chem 1"
# 6. outfile_name = "MJ_Chem_1_040417_1242"
# 7. data_file = "Generic_Array.txt"
# 8. dataColumn = "2"

args <- commandArgs(F);
# scriptPath <- normalizePath(dirname(sub("^--file=", "", args[grep("^--file=", args)])))
scriptPath <- dirname(sub("^--file=", "", args[grep("^--file=", args)]))
args <- commandArgs(TRUE);

if(length(args) >= 9) {
    args;
    length(args);
    workPath <- args[1];
    sub_dir <- args[2];
    platform <- args[3];
    pc_title <- args[4];
    test_chemical <- args[5];
    outfile_name <- args[6];
    data_file <- args[7];
    classifier_version <- args[8];
    number <- as.numeric(args[length(args)]);
    dataColumn <- as.integer(number);

    log_file <- paste(workPath, "logs", "tg_classifier.log", sep = "/");
    print(log_file);
    cat(format(Sys.time(), "# %a %b %d %X %Y"), test_chemical, file = log_file, sep = ", ", append = TRUE);
    cat("\nNumber of arguments: ", length(args), "\n", file = log_file, sep = "", append = TRUE);

    # default directories;
    data_dir = "data";
    classifier_dir = "classifier";
    code_dir = "code";
    results_dir = "results";

    # Modified based on original

    # path to codes
    code_path <- paste(scriptPath, code_dir, sep = "/");

    # path to Classifier, Annotation and Heatmap
    classifier_path <- paste(scriptPath, classifier_dir, classifier_version, sub_dir,  sep = "/")
    print(paste("Classifier Path:", classifier_path, ".\n", sep = ''))
    # path to data file
    data_path <- paste(workPath, data_dir, sub_dir, sep = "/")

    # path to results
    results_path <- paste(workPath, results_dir, sub_dir, sep = "/")

    # Load Classifier
    classifier <- read.table(paste(classifier_path, "Classifier.txt", sep = "/"), 
                                   header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")

    # Load Heatmap
    heat <- read.table(paste(classifier_path, "Heatmap_Data.txt", sep = "/"), 
                             header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")

    # import functions
    source(paste(code_path, "RCode_Functions.R", sep = "/"))

    if(platform == "general") { # The only case
        cat("Platform: ", platform, "\n", file = log_file, sep = "", append = TRUE);
        # Read data file
        # cat("Data Column input: ", args[length(args)], "\n", file=log_file, sep="", append=TRUE);
        cat("Data Column: ", dataColumn, "\n", file = log_file, sep = "", append = TRUE);
        data <- read.table(paste(data_path, data_file, sep = "/"), header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        data[,-1] <- log10(2 ^ data[,-1]) # Convert from log2 to log10
        data[,1] <- toupper(data[,1])
        names(data)[1] <- "Probe"
        results <- General(data[,c(1,dataColumn)], classifier, heat, pc_title, test_chemical, outfile_name)
    }

    param_file <- paste(outfile_name, 'txt', sep = '.')
    write.table(results, file = paste(results_path, param_file, sep = "/"), col.names = NA, append = FALSE, quote = FALSE, sep = "\t")
} else {
    cat("Not enough parameters from command line.\n", file = log_file, append = TRUE);
}
