# Default directories under based dir: Classifier, Code, Data, Results
#
# Arugment parameters with example values:
# 1. workPath
# 2. sub_dir
# 3. platform
# 4. pc_title
# 5. test_chemical
# 6. outfile_name
# 7. treatment_file
# 8. control_file
# 9. dataColumn

args <- commandArgs(F);
# scriptFile <- sub("^--file=", "", args[grep("^--file=", args)])
# print(paste('Full file path: ', scriptFile, '\n', sep =))
# scriptPath <- normalizePath(dirname(sub("^--file=", "", args[grep("^--file=", args)])))
scriptPath <- dirname(sub("^--file=", "", args[grep("^--file=", args)]))
print(paste("Script path: ", scriptPath))
print(args)
args <- commandArgs(TRUE);

if(length(args) >= 8) {
    args;
    length(args);
    workPath <- args[1];
    sub_dir <- args[2];
    platform <- args[3];
    pc_title <- args[4];
    test_chemical <- args[5];
    outfile_name <- args[6];
    treatment_file <- args[7];
    classifier_version <- args[8];
    log_file <- paste(workPath, "logs", "tg_classifier.log", sep = "/");
    cat(format(Sys.time(), "# %a %b %d %X %Y"), test_chemical, file = log_file, sep = ", ", append = TRUE);
    cat("\nNumber of arguments: ", length(args), "\n", file = log_file, sep = "", append = TRUE);

    if(length(args) > 8) {
        control_file <- args[9];
    }

    #default directories;
    data_dir = "data";
    classifier_dir = "classifier";
    code_dir = "code";
    results_dir = "results";
    #.libPaths(c("/home/cebs/R/library", "/usr/lib64/R/library"))

    # Modified based on original

    # path to codes
    code_path <- paste(scriptPath, code_dir, sep="/");

    # path to Classifier, Annotation and Heatmap
    classifier_path <- paste(scriptPath, classifier_dir, classifier_version, sub_dir, sep = "/")
    print(paste("Classifier Path:", classifier_path, ".\n", sep = ''))

    # path to data file
    data_path <- paste(workPath, data_dir, sub_dir, sep = "/")

    # path to results
    results_path <- paste(workPath, results_dir, sub_dir, sep = "/")

    if (!dir.exists(results_path)) {
      dir.create(results_path, recursive = TRUE)
    }

    # Load Classifier
    classifier <- read.table(paste(classifier_path, "Classifier.txt", sep = "/"), 
                                   header = TRUE, sep = "\t", quote = "", 
                                   fill = TRUE, comment.char = "")

    # Load Heatmap
    heat <- read.table(paste(classifier_path, "Heatmap_Data.txt", sep = "/"), 
                             header = TRUE, sep = "\t", quote = "", 
                             fill = TRUE, comment.char = "")

    # import functions
    source(paste(code_path, "RCode_Functions.R", sep = "/"))

    if (platform == "affy") {
        cat("Platform: ", platform, "\n", file = log_file, sep = "", append = TRUE);
        # Load Annotation and keep only probe id and gene symbol columns
        anno <- read.table(paste(classifier_path, "GPL570-13270_Affy_Human.txt", sep = "/"), 
                                 header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        anno[,1] <- paste("X", anno[,1], sep = "")
        anno <- anno[,c(1,11)]
        names(anno) <- c("ProbeID", "GeneSymbol")

        # Perform Affymetrix Analysis
        results <- Affy(treatment_file, control_file, anno, classifier, heat, pc_title, test_chemical, outfile_name)
    } else if (platform == "agilent_dye_swap") {
        cat("Platform: ", platform, "\n", file=log_file, sep="", append=TRUE);
        # Load Annotation
        anno <- read.table(paste(classifier_path, "Agilent_annotation.txt", sep = "/"), 
                                 header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        anno <- anno[,c(1,3)]
        results <- AgilentDyeSwap(treatment_file, control_file, anno, classifier, heat, pc_title, test_chemical, outfile_name)
    } else if (platform == "agilent_one_color") {
        cat("Platform: ", platform, "\n", file=log_file, sep = "", append = TRUE);

        # Load Annotation
        annotation_file <- "GPL7294-9579.txt"
        anno <- read.table(paste(classifier_path, annotation_file, sep = "/"), 
                                 header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        anno <- anno[,c(1,7)]
        anno[,2] <- toupper(anno[,2])
        names(anno) <- c("ProbeID", "GeneSymbol")
        results <- AgilentOneCol(treatment_file, control_file, anno, classifier, heat, pc_title, test_chemical, outfile_name)
    } else if (platform == "agilent_cy5") {
        cat("Platform: ", platform, "\n", file = log_file, sep = "", append = TRUE);
        # Load Annotation
        anno <- read.table(paste(classifier_path, "Agilent_annotation.txt", sep = "/"), 
                                 header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        anno <- anno[,c(1,3)]
        results <- AgilentTwoColCy5(treatment_file, anno, classifier, heat, pc_title, test_chemical, outfile_name)
    } else if (platform == "agilent_cy3") {
        cat("Platform: ", platform, "\n", file=log_file, sep="", append=TRUE);
        # Load Annotation
        anno <- read.table(paste(classifier_path, "Agilent_annotation.txt", sep = "/"), 
                                header = TRUE, sep = "\t", quote = "", fill = TRUE, comment.char = "")
        anno <- anno[,c(1,3)]
        results <- AgilentTwoColCy5(treatment_file, anno, classifier, heat, pc_title, test_chemical, outfile_name)
    } else if (platform == "general") {
        cat("Platform: ", platform, "\n", file=log_file, sep="", append=TRUE);
        # Read data file
        data_file <- treatment_file
        dataColumn <- 2;
        # cat("Data Column input: ", args[length(args)], "\n", file=log_file, sep="", append=TRUE);

        if (length(args) > 6) {
            number <- as.numeric(args[length(args)]);
            dataColumn <- as.integer(number);
        }

        cat("Data Column: ", dataColumn, "\n", file = log_file, sep = "", append = TRUE);
        infile <- paste(data_path, data_file, sep = "/")
        print(paste('Data file: ', infile, ".\n", sep = ''))
        data <- read.table(infile, header = TRUE, sep = "\t", quote = "",fill = TRUE, comment.char = "")
        data[,-1] <- log10(2 ^ data[,-1])
        data[,1] <- toupper(data[,1])
        names(data)[1] <- "Probe"
        results <- General(data[,c(1,dataColumn)], classifier, heat, pc_title, test_chemical, outfile_name)
    }

    param_file <- paste(outfile_name, 'txt', sep = '.')
    write.table(results, file = paste(results_path, param_file, sep = "/"), col.names = NA, append = FALSE, quote = FALSE, sep = "\t")
} else {
    cat("Not enough parameters from command line.\n", file = log_file, append = TRUE);
}
